/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2000 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

#include "httpd.h"
#include "ap_buckets.h"
#include <stdlib.h>

void ap_heap_setaside(ap_bucket *e);

static apr_status_t transient_get_str(ap_bucket *e, const char **str, 
                                      apr_ssize_t *len, int block)
{
    ap_bucket_transient *b = (ap_bucket_transient *)e->data;
    *str = b->start;
    *len = e->length; 
    return APR_SUCCESS;
}

static apr_status_t transient_split(ap_bucket *e, apr_off_t nbyte)
{
    ap_bucket *newbuck;
    ap_bucket_transient *a = (ap_bucket_transient *)e->data; 
    ap_bucket_transient *b; 
    apr_ssize_t dump;

    newbuck = ap_bucket_transient_create(a->start, e->length, &dump);
    b = (ap_bucket_transient *)newbuck->data;

    b->end = a->end;
    a->end = a->start + nbyte;
    b->start = a->end + 1; 

    newbuck->length = e->length - nbyte;
    e->length = nbyte;

    newbuck->prev = e;
    newbuck->next = e->next;
    e->next = newbuck;


    return APR_SUCCESS;
}

/*
 * save nbyte bytes to the bucket.
 * Only returns fewer than nbyte if an error ocurred.
 * Returns -1 if no bytes were written before the error ocurred.
 * It is worth noting that if an error occurs, the buffer is in an unknown
 * state.
 */
static apr_status_t transient_insert(ap_bucket *e, const char *buf,
                               apr_size_t nbyte, apr_ssize_t *w)
{
    ap_bucket_transient *b = (ap_bucket_transient *)e->data;

    if (nbyte == 0) {
        *w = 0;
        return APR_SUCCESS;
    }

    /* We should probably do some checking to make sure we don't allocate too
     * much memory, but that can wait for the second pass.
     */
    b->start = buf;
    b->end = b->start + nbyte;
    *w = nbyte;
    return APR_SUCCESS;
}

API_EXPORT(ap_bucket *) ap_bucket_transient_create(const char *buf,
                               apr_size_t nbyte, apr_ssize_t *w)
{
    ap_bucket *newbuf;
    ap_bucket_transient *b;

    newbuf                = calloc(1, sizeof(*newbuf));
    b                     = malloc(sizeof(*b)); 

    b->start = b->end     = NULL;

    newbuf->data          = b;
    transient_insert(newbuf, buf, nbyte, w);
    newbuf->length        = b->end - b->start;

    newbuf->type          = AP_BUCKET_TRANSIENT;
    newbuf->read          = transient_get_str;
    newbuf->setaside      = ap_heap_setaside;
    newbuf->split         = transient_split;
    newbuf->destroy       = NULL;
    return newbuf;
}

